/* Sensory Confidential
 * Copyright (C)2016-2017 Sensory, Inc. http://sensory.com/
 *
 * Android AudioRecord to read-only SnsrStream adapter.
 *------------------------------------------------------------------------------
 */

package com.sensory.speech.snsr;

import java.io.IOException;

import android.media.AudioFormat;
import android.media.AudioRecord;
import android.media.MediaRecorder;
import android.util.Log;

import com.sensory.speech.snsr.SnsrStream;


/*
 * Implements the SnsrStream.Provider interface for live audio.
 *
 * Create a new SnsrStream instance with:
 * SnsrStream a = SnsrStream.fromProvider(new SnsrStreamAudioDeviceAndroid(16000),
 *                                      SnsrStreamMode.READ);
*/
class SnsrStreamAudioDeviceAndroid implements SnsrStream.Provider {
    private static final String TAG = "SnsrStreamAudioDeviceAndroid";
    private static final int CHANNELS = AudioFormat.CHANNEL_IN_MONO;
    private static final int ENCODING = AudioFormat.ENCODING_PCM_16BIT;
    private static final int SOURCE =
      MediaRecorder.AudioSource.VOICE_RECOGNITION;
    /* Use an audio buffer that is at least this long */
    private static final int MIN_BUFFER_SIZE_MS = 1000;
    private AudioRecord mAudio;
    private int mBufferSize;
    private int mSampleRate;

    /**
     * Static constructor.
     * @param[in] sampleRate the sample rate. Use 16000.
     */
    public SnsrStreamAudioDeviceAndroid(int sampleRate) {
        double minBufferSize = (double)sampleRate * MIN_BUFFER_SIZE_MS / 1000;
        mBufferSize =
          AudioRecord.getMinBufferSize(sampleRate, CHANNELS, ENCODING);
        if (mBufferSize < minBufferSize) {
          mBufferSize =
            mBufferSize * (int)Math.ceil(minBufferSize / mBufferSize);
        }
        mSampleRate = sampleRate;
        Log.d(TAG, String.format("mBufferSize = %d (%.3f seconds)",
                                 mBufferSize, (double)mBufferSize/sampleRate));
    }

    @Override
    public long onOpen() throws IOException {
        Log.d(TAG, "onOpen");
        mAudio = new AudioRecord(SOURCE, mSampleRate,
                                 CHANNELS, ENCODING, mBufferSize);
        if (mAudio == null ||
            mAudio.getState() != AudioRecord.STATE_INITIALIZED) {
          Log.e(TAG, "Could not initialize audio device at " +
                mSampleRate + " Hz.");
          mAudio = null;
          return NOT_OPEN;
        }
        mAudio.startRecording();
        return OK;
    }

    @Override
    public long onClose() throws IOException {
        Log.d(TAG, "onClose");
        mAudio.stop();
        mAudio.release();
        mAudio = null;
        return OK;
    }

    @Override
    public void onRelease() {
        Log.d(TAG, "onRelease");
        if (mAudio != null) mAudio.release();
        mAudio = null;
    }

    @Override
    public long onRead(byte[] buffer) throws IOException {
      int read = mAudio.read(buffer, 0, buffer.length);
      if (Thread.interrupted()) return INTERRUPTED;
      if (read == AudioRecord.ERROR_BAD_VALUE) return INVALID_ARG;
      else if (read < 0) return ERROR;
      return read;
    }

    @Override
    public long onWrite(byte[] buffer) throws IOException {
      return NOT_IMPLEMENTED;
    }
}
